@@ -1,5 +1,6 @@
 require 'json-schema/attribute'
-require 'addressable/uri'
+require 'json-schema/errors/uri_error'
+
 module JSON
   class Schema
     class UriFormat < FormatAttribute
@@ -7,8 +8,8 @@ module JSON
         return unless data.is_a?(String)
         error_message = "The property '#{build_fragment(fragments)}' must be a valid URI"
         begin
-          Addressable::URI.parse(data)
-        rescue Addressable::URI::InvalidURIError
+          JSON::Util::URI.parse(data)
+        rescue JSON::Schema::UriError
           validation_error(processor, error_message, fragments, current_schema, self, options[:record_errors])
         end
       end
